import psycopg2
import psycopg2.extras # For dictionary cursor
import logging
import secrets
import string
from .db_core import get_db_connection # Import from new core module

# Configure basic logging for this module
logger = logging.getLogger(__name__)

TOKEN_LENGTH = 32 # Length for RSS tokens

def _generate_secure_token(length=TOKEN_LENGTH):
    """Generates a cryptographically secure URL-safe token."""
    # Use URL-safe characters
    alphabet = string.ascii_letters + string.digits + '-_'
    token = ''.join(secrets.choice(alphabet) for i in range(length))
    return token

def get_or_create_rss_token(user_id: int) -> str | None:
    """Gets the user's RSS token, generating one if it doesn't exist."""
    token = None
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cursor.execute("SELECT rss_token FROM users WHERE id = %s", (user_id,))
            row = cursor.fetchone()
            if row and row['rss_token']:
                token = row['rss_token']
            else:
                # Generate a new token and ensure it's unique
                while True:
                    new_token = _generate_secure_token()
                    # Check if token already exists (highly unlikely, but good practice)
                    cursor.execute("SELECT 1 FROM users WHERE rss_token = %s", (new_token,))
                    existing = cursor.fetchone()
                    if not existing:
                        cursor.execute("UPDATE users SET rss_token = %s WHERE id = %s", (new_token, user_id))
                        token = new_token
                        break # Exit loop once unique token is set
    except Exception as e:
        logger.error(f"Failed to get or create RSS token for user {user_id}: {e}", exc_info=True)
        # Rollback might be needed if commit failed mid-update
    return token

def regenerate_rss_token(user_id: int) -> str | None:
    """Generates a new RSS token for the user, invalidating the old one."""
    # This function essentially does the same as get_or_create if token is null,
    # but forces generation of a *new* token if one already exists.
    new_token = None
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor() # No DictCursor needed
            while True:
                new_token = _generate_secure_token()
                cursor.execute("SELECT 1 FROM users WHERE rss_token = %s", (new_token,))
                existing = cursor.fetchone()
                if not existing:
                    cursor.execute("UPDATE users SET rss_token = %s WHERE id = %s", (new_token, user_id))
                    break
    except Exception as e:
        logger.error(f"Failed to regenerate RSS token for user {user_id}: {e}", exc_info=True)
        new_token = None # Ensure None is returned on error
    return new_token
